<?php

namespace Drupal\teamleader\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\teamleader\Exception\TeamleaderCredentialsException;
use Drupal\teamleader\TeamleaderApiServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Teamleader settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  use MessengerTrait;

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'teamleader_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['teamleader.settings'];
  }

  /**
   * TeamleaderSettingsForm constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\teamleader\TeamleaderApiServiceInterface $teamleaderApi
   *   The Teamleader API service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    protected TeamleaderApiServiceInterface $teamleaderApi
  ) {
    parent::__construct($config_factory);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('teamleader_api'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): TrustedRedirectResponse|array {
    $form = parent::buildForm($form, $form_state);

    $connection = NULL;
    $connected = FALSE;
    try {
      $connection = $this->teamleaderApi->getConnection();
    }
    catch (TeamleaderCredentialsException $e) {
    }

    // Handle coming back from TL after approval.
    if ($this->getRequest()->query->get('code')) {
      try {
        $this->teamleaderApi->startAuthorization();
        return new TrustedRedirectResponse($this->teamleaderApi->getRedirectUri());
      }
      catch (\Exception $e) {
        $this->messenger()->addError($e->getMessage());
      }
    }

    if ($connection && $connection->getAccessToken()) {
      $this->messenger()->addStatus($this->t('Access token is present. Teamleader is connected'));
      $connected = TRUE;
    }

    $form['actions']['submit']['#value'] = $this->t('Connect to Teamleader');

    $form['credentials'] = [
      '#type' => 'details',
      '#title' => $this->t('Credentials'),
      '#open' => TRUE,
      '#description' => $this->t(
        'To retrieve your client ID & secret: <ol>
          <li>Visit <a href="@url">Teamleader Marketplace</a> and create a new integration.</li>
          <li>Retrieve your client ID/Secret under the "OAuth2 credentials" header.</li>
          </ol>',
        [
          '@url' => 'https://marketplace.teamleader.eu/be/en/build',
        ]
      ),
    ];

    $form['credentials']['client_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client ID'),
      '#disabled' => $connected ?: FALSE,
      '#default_value' => $this->config('teamleader.settings')->get('credentials.client_id'),
      '#required' => TRUE,
      '#size' => 100,
      '#maxlength' => 150,
      '#description' => $this->t('The Teamleader integration client ID.'),
    ];

    $form['credentials']['client_secret'] = [
      '#type' => 'textfield',
      '#disabled' => $connected ?: FALSE,
      '#title' => $this->t('Client secret'),
      '#default_value' => $this->config('teamleader.settings')->get('credentials.client_secret'),
      '#required' => TRUE,
      '#description' => $this->t('The Teamleader integration Client secret.'),
    ];

    if ($connected) {
      $form['actions']['submit']['#access'] = FALSE;
      $form['actions']['disconnect'] = [
        '#type' => 'submit',
        '#value' => 'Disconnect',
        '#submit' => ['::clearAccessTokens'],
      ];
    }

    return $form;
  }

  /**
   * Clears the current TL access tokens.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function clearAccessTokens(array &$form, FormStateInterface $form_state): void {
    $this->teamleaderApi->getConnection()->clearTokens();
    $this->messenger()->addMessage($this->t('Tokens successfully cleared'));
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    parent::submitForm($form, $form_state);

    $this->config('teamleader.settings')
      ->set('credentials.client_id', $form_state->getValue('client_id'))
      ->set('credentials.client_secret', $form_state->getValue('client_secret'))
      ->save();

    try {
      $this->teamleaderApi->startAuthorization();
    }
    catch (\Exception $e) {
      $this->messenger()->addError($e->getMessage());
    }
  }

}
