<?php

namespace Drupal\teamleader;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\GeneratedUrl;
use Drupal\Core\Routing\UrlGeneratorInterface;
use Drupal\Core\State\StateInterface;
use Drupal\teamleader\Exception\TeamleaderCredentialsException;
use Teamleader\Client;
use Teamleader\Connection;

/**
 * Teamleader Integration service.
 *
 * @mixin \Teamleader\Client
 */
class TeamleaderApiService implements TeamleaderApiServiceInterface {

  /**
   * Teamleader connection instance.
   *
   * @var \Teamleader\Connection|null
   */
  protected $connection = NULL;

  /**
   * Teamleader client instance.
   *
   * @var \Teamleader\Client|null
   */
  protected ?Client $client = NULL;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\Core\Routing\UrlGeneratorInterface $urlGenerator
   *   The url generator.
   * @param \Drupal\Core\State\StateInterface $state
   *   The drupal state api.
   */
  public function __construct(
    protected readonly ConfigFactoryInterface $configFactory,
    protected readonly UrlGeneratorInterface $urlGenerator,
    protected readonly StateInterface $state,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getConnection(): Connection {
    if (!$this->connection) {
      $settings = $this->configFactory->get('teamleader.settings');
      if (!$settings->get('credentials.client_id')) {
        throw new TeamleaderCredentialsException('Teamleader credentials are missing.');
      }

      $cache_handler = new TeamleaderApiCacheHandler($this->state);
      $this->connection = new Connection($cache_handler);
      $this->connection->setClientId($settings->get('credentials.client_id'));
      $this->connection->setClientSecret($settings->get('credentials.client_secret'));
      $this->connection->setRedirectUrl($this->getRedirectUri());
    }

    return $this->connection;
  }

  /**
   * {@inheritdoc}
   */
  public function getClient(): Client {
    if (!$this->client) {
      $this->client = new Client($this->getConnection());
    }

    return $this->client;
  }

  /**
   * {@inheritdoc}
   */
  public function startAuthorization(): void {
    $connection = $this->getConnection();
    $connection->acquireAccessToken();
  }

  /**
   * {@inheritdoc}
   */
  public function getRedirectUri(): GeneratedUrl|string {
    return $this->urlGenerator->generateFromRoute('teamleader.settings_form', [], [
      'absolute' => TRUE,
    ]);
  }

  /**
   * Pass calls to the TL client.
   *
   * @param string $name
   *   TL client Method name.
   * @param mixed $arguments
   *   TL client Method arguments.
   *
   * @return \Teamleader\Model
   *   The TL model to use.
   *
   * @throws \Drupal\teamleader\Exception\TeamleaderCredentialsException
   * @throws \Exception
   */
  public function __call(string $name, $arguments) {
    $client = $this->getClient();

    if (method_exists($client, $name)) {
      return call_user_func_array([$client, $name], $arguments);
    }

    throw new \Exception(sprintf('Method %s not found', $name));
  }

}
